@@ -27,3 +27,8 @@ mstr <- future.BatchJobs:::mstr
 printf <- future.BatchJobs:::printf
 tempRegistry <- future.BatchJobs:::tempRegistry
 trim <- future.BatchJobs:::trim
+attachLocally <- function(x, envir=parent.frame()) {
+  for (name in names(x)) {
+    assign(name, value=x[[name]], envir=envir)
+  }
+}
